require ("Player")
require ("GhostPlayer")
require ("Timer")
require ("Level")

function love.load()
	player = Player:new()
	timer = Timer:new()
	timer:setCompletion(timerFinished)
	gameOver = false
	titleScreen = true
	titleScreenTimer = 255
	titleScreenTimerDirection = -5
	if love.web then
		smallFont = love.graphics.newFont(36)
	else
		bigFont = love.graphics.newFont("gfx/kenpixel_blocks.ttf", 36)
	end
	if love.web then
		smallFont = love.graphics.newFont(20)
	else
		smallFont = love.graphics.newFont("gfx/kenpixel_blocks.ttf", 20)
	end
    music = love.audio.newSource("audio/bu-the-goats-retort.ogg")
    music:setLooping(true)
    totalTime = 0
end

function startGame()
	level = Level:new(1)
	ghostPlayers = {}
	player.spawnPoint = level:getSpawnPoint()
	player:reset()
	love.audio.play(music)
end

function love.update(dt)
	if gameOver or titleScreen then
		return
	end
	totalTime = totalTime + 1 * dt
	-- if love.keyboard.isDown("z") then 
	-- 	timer.time = 0
	-- 	return
	-- end
	timer:update(dt)
	level:resetPressurePoints()
	for i, ghostPlayer in ipairs(ghostPlayers) do
		ghostPlayer:move()
		level:addPressurePoint({x = ghostPlayer.position.x + ghostPlayer.size.width / 2, y = ghostPlayer.position.y + ghostPlayer.size.height / 2})
	end
	if player then
		local newPosition = {x = player.position.x, y = player.position.y}
		if level:checkCollision({x = player.position.x + player.size.width / 2, y = player.position.y + player.size.height / 2}) then
			return
		end
		if love.keyboard.isDown("right") or love.keyboard.isDown("d") then
			newPosition.x = newPosition.x + player.speed * dt
			while (level:checkCollision({x = newPosition.x + player.size.width - player.inset - 1, y = newPosition.y + player.inset}) or level:checkCollision({x = newPosition.x + player.size.width - player.inset - 1, y = newPosition.y + player.size.height - player.inset - 1})) and newPosition.x > player.position.x do
				newPosition.x = math.ceil(newPosition.x) - 1
			end
		elseif love.keyboard.isDown("left") or love.keyboard.isDown("a") then
			newPosition.x = newPosition.x - player.speed * dt
			while (level:checkCollision({x = newPosition.x + player.inset, y = newPosition.y + player.inset}) or level:checkCollision({x = newPosition.x + player.inset, y = newPosition.y + player.size.height - player.inset - 1})) and newPosition.x < player.position.x do
				newPosition.x = math.floor(newPosition.x) + 1
			end
		end
		if love.keyboard.isDown("down") or love.keyboard.isDown("s") then
			newPosition.y = newPosition.y + player.speed * dt
			while (level:checkCollision({x = newPosition.x + player.inset, y = newPosition.y + player.size.height - player.inset - 1}) or level:checkCollision({x = newPosition.x + player.size.width - player.inset - 1, y = newPosition.y + player.size.height - player.inset - 1})) and newPosition.y > player.position.y do
				newPosition.y = math.ceil(newPosition.y) - 1
			end
		elseif love.keyboard.isDown("up") or love.keyboard.isDown("w") then
			newPosition.y = newPosition.y - player.speed * dt
			while (level:checkCollision({x = newPosition.x + player.inset, y = newPosition.y + player.inset}) or level:checkCollision({x = newPosition.x + player.size.width - player.inset - 1, y = newPosition.y + player.inset})) and newPosition.y < player.position.y do
				newPosition.y = math.floor(newPosition.y) + 1
			end
		end
		player:move(newPosition)
		level:addPressurePoint({x = newPosition.x + player.size.width / 2, y = newPosition.y + player.size.height / 2})
		if level:isGoalTile({x = newPosition.x + player.size.width / 2, y = newPosition.y + player.size.height / 2}) then
			level = level:nextLevel();
			if level then
				ghostPlayers = {}
				player.spawnPoint = level:getSpawnPoint()
				player:reset()
				timer:reset()
				local sound = love.audio.newSource("audio/win.wav", "static")
				love.audio.play(sound)
			else 
				gameOver = true
				love.audio.stop(music)
			end
		end
	end
end

function love.draw()
	if gameOver then
		love.graphics.setColor(255, 255, 255)
    	love.graphics.setFont(bigFont)
		love.graphics.printf("A winner is you\nTotal Time: " .. math.floor(totalTime), 0, love.graphics.getHeight() / 2  - bigFont:getHeight() / 2, love.graphics.getWidth(), "center")
	elseif titleScreen then
		love.graphics.setColor(255, 255, 255)
    	love.graphics.setFont(bigFont)
		love.graphics.printf("TIME GHOST", 0, love.graphics.getHeight() * 0.25, love.graphics.getWidth(), "center")
    	love.graphics.setFont(smallFont)
		love.graphics.printf("(a Ludum Dare 27 game)", 0, love.graphics.getHeight() * 0.25 + bigFont:getHeight() , love.graphics.getWidth(), "center")
		love.graphics.setColor(255, 255, 255, titleScreenTimer)
		love.graphics.printf("Press any key to start", 0, love.graphics.getHeight() * 0.75, love.graphics.getWidth(), "center")
		titleScreenTimer = titleScreenTimer + titleScreenTimerDirection
		if titleScreenTimer <= 0 then
			titleScreenTimer = 0
			titleScreenTimerDirection = 5
		elseif titleScreenTimer >= 255 then
			titleScreenTimer = 255
			titleScreenTimerDirection = -5
		end
		love.graphics.setColor(255, 255, 255)
		love.graphics.setFont(love.graphics.newFont("gfx/kenpixel_blocks.ttf", 12))
		love.graphics.printf("©2013 Fabio Ritrovato - @Sephiroth87", 0, love.graphics.getHeight() - 20, love.graphics.getWidth(), "center")
	else
		level:draw()
		for i, ghostPlayer in ipairs(ghostPlayers) do
			ghostPlayer.alpha = 200 / #ghostPlayers * i
			ghostPlayer:draw(dt)
		end
		if player then
			player:draw()
		end
		timer:draw()
		love.graphics.setFont(smallFont)
		love.graphics.setColor(255, 255, 255)
		love.graphics.print("Level: " .. level.levelNumber, 10, 10)
	end
end

function love.keypressed(key)
	if titleScreen == true then
		titleScreen = false
		startGame()
	elseif gameOver == true then
		titleScreen = true
		gameOver = false
		titleScreenTimer = 255
		titleScreenTimerDirection = -5
	end
end

function timerFinished()
	for i, ghostPlayer in ipairs(ghostPlayers) do
		ghostPlayer:reset(dt)
	end
	if #ghostPlayers > 0 and #ghostPlayers == level.maxGhosts then
		table.remove(ghostPlayers, 1)
	end
	if #ghostPlayers < level.maxGhosts then
		table.insert(ghostPlayers, GhostPlayer:new(player:getMoves(), level:getSpawnPoint()))
	end
	player:reset()
	timer:reset()
	local sound = love.audio.newSource("audio/teleport.wav", "static")
	love.audio.play(sound)
end